#include "BoundingBox.h"

/*--------------------------------------------------------------------
* create function that takes parameters, with a Vector3d for position
--------------------------------------------------------------------*/
void BoundingBox::create(const float w, const float h, const float d, const Vector3d &pos){
	_position.set(pos);
	init(w, h, d);
}

/*------------------------------------------------------------------------------
* create function that takes parameters, with an x, y, and z specified for position
--------------------------------------------------------------------------------*/
void BoundingBox::create(const float w, const float h, const float d, const double x, const double y, const double z){
	_position.set(x, y, z);
	init(w, h, d);
}

/*-------------------------------------------------------
* common init function used by both create methods - 
* initialises the width, height and depth and sets the 
* Vector3d's for the points and normals on all the faces,
* and calls setBounds();
---------------------------------------------------------*/
void BoundingBox::init(const float w, const float h, const float d){
	//width, height and depth calculations
	_fWidth = w;
	_fHeight = h;
	_fDepth = d;
	_fHalfW = w * 0.5f;
	_fHalfH = h * 0.5f;
	_fHalfD = d * 0.5f;

	//points on the faces
	_minusXPoint.set(_position._x - _fHalfW, _position._y, _position._z);
	_minusYPoint.set(_position._x, _position._y - _fHalfH, _position._z);
	_minusZPoint.set(_position._x, _position._y, _position._z - _fHalfD);
	_plusXPoint.set(_position._x + _fHalfW, _position._y, _position._z);
	_plusYPoint.set(_position._x, _position._y + _fHalfH, _position._z);
	_plusZPoint.set(_position._x, _position._y, _position._z + _fHalfD);

	//normals on the faces
	_minusXNormal.set(-1.0,  0.0,  0.0);
	_minusYNormal.set( 0.0, -1.0,  0.0);
	_minusZNormal.set( 0.0,  0.0, -1.0);
	_plusXNormal.set(1.0, 0.0, 0.0);
	_plusYNormal.set(0.0, 1.0, 0.0);
	_plusZNormal.set(0.0, 0.0, 1.0);

	setBounds(); // hold the bounds
}

/*----------------------------------------------
* move the bounding box
------------------------------------------------*/
void BoundingBox::move(const double x, const double y, const double z){
	_position.set(x, y, z);
	setBounds(); // hold the bounds
}

/*--------------------------------------
* move the bounding box using a Vector3d
----------------------------------------*/
void BoundingBox::move(const Vector3d &pos){
	_position.set(pos);
	setBounds(); // hold the bounds
}

/*------------------------------------------
* set the six bounds of the boundingbox
--------------------------------------------*/
void BoundingBox::setBounds(){
	_minusXBound = _position._x - _fHalfW;
	_plusXBound = _position._x + _fHalfW;
	_minusYBound = _position._y - _fHalfH;
	_plusYBound = _position._y + _fHalfH;
	_minusZBound = _position._z - _fHalfD;
	_plusZBound = _position._z + _fHalfD;
}

/*----------------------------------------------------------
* builds the BoundingBox - draws one quad per side
------------------------------------------------------------*/
void BoundingBox::drawBox() const{
	// Front Face
	glBegin(GL_LINE_LOOP);
		glNormal3f(0.0f,0.0f,1.0f);
		glVertex3d(-_fHalfW, -_fHalfH,  _fHalfD);
		glVertex3d( _fHalfW, -_fHalfH,  _fHalfD);
		glVertex3d( _fHalfW,  _fHalfH,  _fHalfD);
		glVertex3d(-_fHalfW,  _fHalfH,  _fHalfD);
	glEnd();
	// Back Face
	glBegin(GL_LINE_LOOP);
		glNormal3f(0.0f,0.0f,-1.0f);
		glVertex3d(-_fHalfW, -_fHalfH, -_fHalfD);
		glVertex3d(-_fHalfW,  _fHalfH, -_fHalfD);
		glVertex3d( _fHalfW,  _fHalfH, -_fHalfD);
		glVertex3d( _fHalfW, -_fHalfH, -_fHalfD);
	glEnd();
	// Top Face
	glBegin(GL_LINE_LOOP);
		glNormal3f(0.0f,1.0f,0.0f);
		glVertex3d(-_fHalfW,  _fHalfH, -_fHalfD);
		glVertex3d(-_fHalfW,  _fHalfH,  _fHalfD);
		glVertex3d( _fHalfW,  _fHalfH,  _fHalfD);
		glVertex3d( _fHalfW,  _fHalfH, -_fHalfD);
	glEnd();
	// Bottom Face
	glBegin(GL_LINE_LOOP);
		glNormal3f(0.0f,-1.0f,0.0f);
		glVertex3d(-_fHalfW, -_fHalfH, -_fHalfD);
		glVertex3d( _fHalfW, -_fHalfH, -_fHalfD);
		glVertex3d( _fHalfW, -_fHalfH,  _fHalfD);
		glVertex3d(-_fHalfW, -_fHalfH,  _fHalfD);
	glEnd();
	// Left face
	glBegin(GL_LINE_LOOP);
		glNormal3f(1.0f,0.0f,0.0f);
		glVertex3d( _fHalfW, -_fHalfH, -_fHalfD);
		glVertex3d( _fHalfW,  _fHalfH, -_fHalfD);
		glVertex3d( _fHalfW,  _fHalfH,  _fHalfD);
		glVertex3d( _fHalfW, -_fHalfH,  _fHalfD);
	glEnd();
	// Right Face
	glBegin(GL_LINE_LOOP);
		glNormal3f(-1.0f,0.0f,0.0f);
		glVertex3d(-_fHalfW, -_fHalfH, -_fHalfD);
		glVertex3d(-_fHalfW, -_fHalfH,  _fHalfD);
		glVertex3d(-_fHalfW,  _fHalfH,  _fHalfD);
		glVertex3d(-_fHalfW,  _fHalfH, -_fHalfD);
	glEnd();
}

/*-------------------------------------
* the draw function - call drawBox()
---------------------------------------*/
void BoundingBox::draw() const{
	glPushMatrix();
		glTranslated(_position._x, _position._y, _position._z);
		drawBox();
	glPopMatrix();
}