#pragma once

#include "GXBase.h"
#include "Vector3d.h"

class BoundingBox{
private:
	float _fWidth; //width of the BoundingBox
	float _fHeight; //height of the BoundingBox
	float _fDepth; //depth of the BoundingBox
	float _fHalfW, _fHalfH, _fHalfD; // half the width height and depth
	double _minusXBound, _plusXBound, _minusYBound, _plusYBound, _minusZBound, _plusZBound; // bounds for the six sides of the box
	Vector3d _minusXPoint, _plusXPoint, _minusYPoint, _plusYPoint, _minusZPoint, _plusZPoint; // points on all the faces (used for collision detection)
	Vector3d _minusXNormal, _plusXNormal, _minusYNormal, _plusYNormal, _minusZNormal, _plusZNormal; // normals on all the faces (collision detection)
	Vector3d _position; // position
	
	//build the BoundingBox
	void drawBox() const;

	// common init function used by both create methods - initialises the width, height and depth
	// and sets the Vector3d's for the points and normals on all the faces
	// and calls setBounds()
	void init(const float w, const float h, const float d);

public:
	// create function that takes parameters, with a Vector3d for position
	void create(const float w, const float h, const float d, const Vector3d &pos);

	// create function that takes parameters, with an x, y, and z specified for position
	void create(const float w, const float h, const float d, const double x, const double y, const double z);

	// move the bounding box
	void move(const double x, const double y, const double z);

	// move the bounding box using a Vector3d
	void move(const Vector3d &pos);

	//set the six bounds of the boundingbox
	void setBounds();

	//get the different bounds of the box
	const double getMinusXBound() const { return _minusXBound; };
	const double getMinusYBound() const { return _minusYBound; };
	const double getMinusZBound() const { return _minusZBound; };
	const double getPlusXBound()  const { return _plusXBound;  };
	const double getPlusYBound()  const { return _plusYBound;  };
	const double getPlusZBound()  const { return _plusZBound;  };

	//get the different points on the faces - returning pointers as they're used
	// alot in the PhysicsIF class
	const Vector3d* getMinusXPoint() const { return &_minusXPoint; };
	const Vector3d* getMinusYPoint() const { return &_minusYPoint; };
	const Vector3d* getMinusZPoint() const { return &_minusZPoint; };
	const Vector3d* getPlusXPoint()  const { return &_plusXPoint;  };
	const Vector3d* getPlusYPoint()  const { return &_plusYPoint;  };
	const Vector3d* getPlusZPoint()  const { return &_plusZPoint;  };

	//get the different normals on the faces - returning pointers as they're used
	// alot in the PhysicsIF class
	const Vector3d* getMinusXNormal() const { return &_minusXNormal; };
	const Vector3d* getMinusYNormal() const { return &_minusYNormal; };
	const Vector3d* getMinusZNormal() const { return &_minusZNormal; };
	const Vector3d* getPlusXNormal()  const { return &_plusXNormal;  };
	const Vector3d* getPlusYNormal()  const { return &_plusYNormal;  };
	const Vector3d* getPlusZNormal()  const { return &_plusZNormal;  };

	//draw it
	void draw() const;
};