#include "BulletHole.h"

/*------------------------------------------------------
* specific constructor
--------------------------------------------------------*/
BulletHole::BulletHole(const Vector3d & center, const Vector3d & normal):_size(0.1f){
	_material.create(Color::black(), Color(0.0f,0.0f,0.0f,1.0f));
	create(center, normal);
};

/*---------------------------------------------
* init the bullet hole with a center, normal
-----------------------------------------------*/
void BulletHole::create(const Vector3d & center, const Vector3d & normal){
	_center.set(center);
	_normal.set(normal);
	_texNum = getTexture("bulletHole.bmp", true); // get the bullethole texture
}

/*-----------------------------------------------------
* draw the bullet hole - fade it out over time
-------------------------------------------------------*/
void BulletHole::draw() const{
	glDepthMask(GL_FALSE); // stop writing to the depth buffer so we don't get z buffer 
						   // contension with other bullet holes
						   // but it'll still work with other geometry

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // enable blending

	_material.apply(); // add the material so we can fade
	glBindTexture(GL_TEXTURE_2D, _texNum); //apply the texture

	glPushMatrix();
		glBegin(GL_QUADS);
			glNormal3d(_normal._x, _normal._y, _normal._z);
			//draw the plane depeding on which way the normal is - there's bound to be a better way to do this tho
			if(_normal._x == 1.0){
				glTexCoord2d(0.0, 0.0);		glVertex3d(_center._x + 0.001, _center._y - _size,  _center._z + _size);
				glTexCoord2d(1.0, 0.0);		glVertex3d(_center._x + 0.001, _center._y - _size,  _center._z - _size);
				glTexCoord2d(1.0, 1.0);		glVertex3d(_center._x + 0.001, _center._y + _size,  _center._z - _size);
				glTexCoord2d(0.0, 1.0);		glVertex3d(_center._x + 0.001, _center._y + _size,  _center._z + _size);
			} else if(_normal._x == -1.0){
				glTexCoord2d(0.0, 0.0);		glVertex3d(_center._x - 0.001, _center._y - _size,  _center._z - _size);
				glTexCoord2d(1.0, 0.0);		glVertex3d(_center._x - 0.001, _center._y - _size,  _center._z + _size);
				glTexCoord2d(1.0, 1.0);		glVertex3d(_center._x - 0.001, _center._y + _size,  _center._z + _size);
				glTexCoord2d(0.0, 1.0);		glVertex3d(_center._x - 0.001, _center._y + _size,  _center._z - _size);
			} else if(_normal._z == 1.0){
				glTexCoord2d(0.0, 0.0);		glVertex3d(_center._x - _size, _center._y - _size,  _center._z + 0.001);
				glTexCoord2d(1.0, 0.0);		glVertex3d(_center._x + _size, _center._y - _size,  _center._z + 0.001);
				glTexCoord2d(1.0, 1.0);		glVertex3d(_center._x + _size, _center._y + _size,  _center._z + 0.001);
				glTexCoord2d(0.0, 1.0);		glVertex3d(_center._x - _size, _center._y + _size,  _center._z + 0.001);
			} else if(_normal._z == -1.0){
				glTexCoord2d(0.0, 0.0);		glVertex3d(_center._x + _size, _center._y - _size,  _center._z - 0.001);
				glTexCoord2d(1.0, 0.0);		glVertex3d(_center._x - _size, _center._y - _size,  _center._z - 0.001);
				glTexCoord2d(1.0, 1.0);		glVertex3d(_center._x - _size, _center._y + _size,  _center._z - 0.001);
				glTexCoord2d(0.0, 1.0);		glVertex3d(_center._x + _size, _center._y + _size,  _center._z - 0.001);
			} else if(_normal._y == 1.0){
				glTexCoord2d(0.0, 0.0);		glVertex3d(_center._x - _size, _center._y + 0.001,  _center._z + _size);
				glTexCoord2d(1.0, 0.0);		glVertex3d(_center._x + _size, _center._y + 0.001,  _center._z + _size);
				glTexCoord2d(1.0, 1.0);		glVertex3d(_center._x + _size, _center._y + 0.001,  _center._z - _size);
				glTexCoord2d(0.0, 1.0);		glVertex3d(_center._x - _size, _center._y + 0.001,  _center._z - _size);
			} else if(_normal._y == -1.0){
				glTexCoord2d(0.0, 0.0);		glVertex3d(_center._x + _size, _center._y - 0.001,  _center._z + _size);
				glTexCoord2d(1.0, 0.0);		glVertex3d(_center._x - _size, _center._y - 0.001,  _center._z + _size);
				glTexCoord2d(1.0, 1.0);		glVertex3d(_center._x - _size, _center._y - 0.001,  _center._z - _size);
				glTexCoord2d(0.0, 1.0);		glVertex3d(_center._x + _size, _center._y - 0.001,  _center._z - _size);
			}
		glEnd();
	glPopMatrix();

	glDisable(GL_BLEND);// disable blending
	glDisable(GL_TEXTURE_2D);

	glDepthMask(GL_TRUE); // enable writing to the depth buffer again
}