#pragma once

#include "Vector3d.h"
#include "GXBase.h"
#include <cmath>

using namespace gxbase;

//camera class loosely based on one from "Computer Graphics using OpenGL" (second edition) by F.S. Hill, Jr.
class Camera{
private:
	Vector3d _eye; // the eye position vector
	Vector3d _lookAt; // the eye lookAt vector
	Vector3d _up; // the eye up vector
	double _dFov, _dAspect, _dNearDist, _dFarDist; // view volume shape
	float _xAngle, _yAngle; // hold the angles passed by the mouses movement for updating the position

public:
	void updatePosition(); // function used by both turn and move

public:
	Camera():_eye(0.0, 0.0, 0.0), _lookAt(0.0, 0.0, 0.0), _up(0.0, 1.0, 0.0){}
	//set the matrix - uses gluLookAt
	void setMatrix() const;

	// set the camera position - similar to gluLookAt
	void setCameraPos(const Vector3d eye);

	//overloaded setCameraPos function that takes the raw coords instead of Vector3d
	void setCameraPos(const double eyex, const double eyey, const double eyez);

	//get the camera eye position
	Vector3d getCameraPos() const { return _eye; };

	//get the camera lookAt vector
	Vector3d getCameraLookAt() const { return _lookAt; };

	//set the camera lookAt vector
	void setCameraLookAt(const Vector3d& pos) { _lookAt.set(pos); };

	//set up the camera - uses gluPerspective
	void setCameraShape(const double fov, const double aspect, const double nearDist, const double farDist);

	// turn the camera. Gets its angles from the x and y of the mouse position
	void turn(const float xAngle, const float yAngle);

	//move the camera along the eye/lookAt vector
	void move(const float amount);

	//move the camera to a specified Vector3d
	void move(const Vector3d& pos);

	//calculate next move, and return the coords - but don't move there yet
	Vector3d calcNextMove(const float amount);
};