#include "Cuboid.h"

/*------------------------------------------------------
* create the cube - initialize the vars and the planes
--------------------------------------------------------*/
void Cuboid::create(const float fWidth, const float fHeight, const float fDepth,
					const GLuint iFrontTexture, const GLuint iBackTexture, const GLuint iLeftTexture,
					const GLuint iRightTexture, const GLuint iTopTexture, const GLuint iBottomTexture,
					const bool repeatTexture){
	//initialize the vars etc
	_fWidth = fWidth;
	_fHeight = fHeight;
	_fDepth = fDepth;
	_iFrontTexture = iFrontTexture;
	_iBackTexture = iBackTexture;
	_iLeftTexture = iLeftTexture;
	_iRightTexture = iRightTexture;
	_iTopTexture = iTopTexture;
	_iBottomTexture = iBottomTexture;

	//initialize the planes, with each plane taking its width and height
	//depending on where it is in the cube
	_pFrontFace.create(_fWidth, _fHeight, _iFrontTexture, repeatTexture); // front face
	_pBackFace.create(_fWidth, _fHeight, _iBackTexture, repeatTexture); // back face
	_pLeftFace.create(_fDepth, _fHeight, _iLeftTexture, repeatTexture); // left face
	_pRightFace.create(_fDepth, _fHeight, _iRightTexture, repeatTexture); // right face
	_pTopFace.create(_fWidth, _fDepth, _iTopTexture, repeatTexture); // top face
	_pBottomFace.create(_fWidth, _fDepth, _iBottomTexture, repeatTexture); //bottom face
}

/*----------------------------------------------------------
* builds the cuboid. moves each face into place, then calls
* the Plane::draw() method to draw it
------------------------------------------------------------*/
void Cuboid::buildCuboid() const{
	float fHalfW = _fWidth * 0.5f;
	float fHalfH = _fHeight * 0.5f;
	float fHalfD = _fDepth * 0.5f;

	//translate and rotate each face into position before calling the draw method on it
	//NOTE - the cubes origin is the bottom left hand corner to make it easier to position,
	//		 so translate and rotate accordingly
	glPushMatrix();
		glTranslatef(fHalfW, fHalfH, _fDepth);
		_pFrontFace.draw(); // front face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(fHalfW, fHalfH, 0.0);
		glRotatef(180, 0.0f, 1.0f, 0.0f);
		_pBackFace.draw(); // back face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(_fWidth, fHalfH, fHalfD);
		glRotatef(90, 0.0f, 1.0f, 0.0f);
		_pLeftFace.draw(); // left face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(0.0, fHalfH, fHalfD);
		glRotatef(-90, 0.0f, 1.0f, 0.0f);
		_pRightFace.draw(); // right face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(fHalfW, _fHeight, fHalfD);
		glRotatef(-90, 1.0f, 0.0f, 0.0f);
		_pTopFace.draw(); // top face
	glPopMatrix();

	glPushMatrix();
		glTranslatef(fHalfW, 0.0, fHalfD);
		glRotatef(90, 1.0f, 0.0f, 0.0f);
		_pBottomFace.draw(); // bottom face
	glPopMatrix();
}

/*-------------------------------------
* the draw function - call buildCuboid
---------------------------------------*/
void Cuboid::draw() const{
	glPushMatrix();
		buildCuboid();
	glPopMatrix();
}