#include "Gun.h"
#include "ObjectMeshManager.h" // include the objectMeshManager so we can add the created bulletlines and bulletholes

/*-----------------------------------------------------------------
* init the gun with a direction, number of bullets, and far plane
-------------------------------------------------------------------*/
void Gun::create(const Vector3d &pos, const int totalBullets, const float farPlane){
	position().set(pos);
	iTotalBullets(totalBullets);
	fFarPlane(farPlane);
}

/*-------------------------------------------------------------------
* receive the bullet direction vector3d from the player class.
* see if any objects collide with the bullet ray
* calls checkBulletCollision() in the PhysicsIF class
---------------------------------------------------------------------*/
void Gun::shoot(const Vector3d & bulletDir){
	if(!bIsShooting() && _iTotalBullets > 0){
		_iTotalBullets--;
		bIsShooting(true); // we're currently shooting, so cancel any extra shoot calls for the time being
		_bullet = bulletDir; // set the bullet vector
		_bullet.Normalise(); // normalise it
		_bHitAnything = checkBulletCollision(this, position(), _bullet, _hitPos, _hitNormal); // check for collision
		// if we hit anything, create a new bullet line
		// the physicsFunction will call the onShot() method for the object hit
		if(_bHitAnything) {
			Object * bl = new BulletLine(position(), _hitPos); // new bullet line
			ObjectMeshManager::get()->addNonStatic(bl);
		} else {
			//fire a bullet into space
			Object * bl = new BulletLine(position(), position() + _bullet * fFarPlane());
			ObjectMeshManager::get()->addNonStatic(bl);
		}
	}
}