#pragma once

#include "PhysicsObject.h"
#include "BulletLine.h"
#include "BulletHole.h"
#include "ParticleEngine.h"

class Gun : public PhysicsObject{
private:
	bool _bIsShooting, _bHitAnything; // is the gun currently shooting, did it hit anything
	int _iTotalBullets; // total number of bullets
	float _fFarPlane; // far plane to check bullets hits with - typically the player's far render plane
	Vector3d _bullet; // a vector3d for the bullet direction
	Vector3d _hitPos, _hitNormal; // vectors to hold the hit position and hit normal of the bullet if it hits anything

public:
	//constructor to initialize the basics
	Gun():_bIsShooting(0), _bHitAnything(0){};

	//init the gun with a direction, number of bullets, and far plane
	void create(const Vector3d & pos, const int totalBullets, const float farPlane);

	//set and get the _bIsMoving bool
	void bIsShooting(const bool b) { _bIsShooting = b; };
	bool bIsShooting() const { return _bIsShooting; };

	int iTotalBullets() const { return _iTotalBullets; }; // return the number of bullets
	void iTotalBullets(const int tb) { _iTotalBullets = tb; }; // set the number of bullets

	float fFarPlane() const { return _fFarPlane; }; // return the far plane
	void fFarPlane(const float fp) { _fFarPlane = fp; }; // set the far plane

	// receive the bullet direction vector3d from the player class.
	// see if any objects collide with the bullet ray
	// calls checkBulletCollision() in the PhysicsIF class
	void shoot(const Vector3d & bulletDir);
	void drawBulletLine();

	//the draw function - (NOT NEEDED FOR GUN?)
	void draw() const { return; };

	//the read function - not used for gun, but needed to instanstiate
	void read(istream &){ return; };

	//update the gun's position to reflect the player
	void updatePosition(const Vector3d & pos){ position().set(pos); };
};