#include "Light.h"

Light::Light() {
	_iLightNumber = 0;
}
Light::~Light(){
	cout << " deleting light object" << endl;
}

/*-----------------------------------------------
* common function used in construction
-------------------------------------------------*/
void Light::create(GLuint lightNum, const Color &ambient, const Color &diffuse, const Vector4f &position) {
	// the light being defined
	_iLightNumber = lightNum;	

	//set the colors
	setAmbient(ambient);
	setDiffuse(diffuse);
	setPosition(position);
	if(position[3] != 0) setSpotlightVars();
	
	/*if(position[3] != 0){
	GLfloat direction[3] = {0.0f, -1.0f, 0.0f};
		GLfloat cutoff = 90.0f;
		GLfloat exp = 1.0f;
		glLightfv(GL_LIGHT0 + _iLightNumber, GL_SPOT_DIRECTION, direction);
		glLightfv(GL_LIGHT0 + _iLightNumber, GL_SPOT_CUTOFF, &cutoff);
		glLightfv(GL_LIGHT0 + _iLightNumber, GL_SPOT_EXPONENT, &exp);
	}*/
}

/*-----------------------------------------------
* setters for the different colors and position
-------------------------------------------------*/
void Light::setAmbient(const Color &ambient) {
	_ambient = ambient;
	glLightfv(GL_LIGHT0 + _iLightNumber, GL_AMBIENT, _ambient.rgba());
}		
void Light::setDiffuse(const Color &diffuse) {
	_diffuse = diffuse;
	glLightfv(GL_LIGHT0 + _iLightNumber, GL_DIFFUSE, _diffuse.rgba());
}		
void Light::setPosition(const Vector4f &position) {
	_position = position;
	glLightfv(GL_LIGHT0 + _iLightNumber, GL_POSITION, _position.xyzw());
}
void Light::setSpotlightVars(){
	glLightfv(GL_LIGHT0 + _iLightNumber, GL_SPOT_DIRECTION, _fDirection);
	glLightfv(GL_LIGHT0 + _iLightNumber, GL_SPOT_CUTOFF, &_fCutoff);
	glLightfv(GL_LIGHT0 + _iLightNumber, GL_SPOT_EXPONENT, &_fExponent);
}

/*-----------------------------
* turn the light on
-------------------------------*/
void Light::apply() const {
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0 + _iLightNumber);
}

/*-------------------------------------------------
* gets and sets for particular variable in position
---------------------------------------------------*/
const float& Light::operator [](int i) const{
	return _position[i];
}
float& Light::operator [](int i){
	return _position[i];
}

/*-------------------------------------------------------------------------
* the read in function - receives the istream from the ObjectMeshManager
---------------------------------------------------------------------------*/
void Light::read(istream &fin) {
	char buffer[256], ch;
	GLfloat r, g, b, a, x, y, z, w; //temps used in setting the color and position up

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _iLightNumber; //first stream in _iLightNumber

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> ch >> r >> ch >> g >> ch >> b >> ch >> a >> ch;
	setAmbient(Color(r, g, b, a)); //streamed in ambient color and set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> ch >> r >> ch >> g >> ch >> b >> ch >> a >> ch;
	setDiffuse(Color(r, g, b, a)); //streamed in diffuse color and set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> ch >> x >> ch >> y >> ch >> z >> ch >> w >> ch;
	setPosition(Vector4f(x, y, z, w)); //streamed in position and set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> ch >> _fDirection[0] >> ch >> _fDirection[1] >> ch >> _fDirection[2] >> ch; // stream in the direction

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fCutoff; // stream in the cutoff

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fExponent; // stream in the exponent

	create(_iLightNumber, _ambient, _diffuse, _position);
}

/*--------------------------------
* the write function
----------------------------------*/
ostream& Light::write(ostream &out){
	out << "Light {"
		 << "\n\tnumber= " << _iLightNumber
		 << "\n\tambient= " << _ambient
		 << "\n\tdiffuse= " << _diffuse
		 << "\n\tposition= " << _position
		 << "\n}" << endl;
	return out;
}

/*-------------------------------------
* update the light
---------------------------------------*/
void Light::update(){
	setPosition(_position);
	glLightfv(GL_LIGHT0 + _iLightNumber, GL_SPOT_DIRECTION, _fDirection);
}

/*----------------------------------------------
* draw function called from ObjectMeshManager
------------------------------------------------*/
void Light::draw() const {
	//return;
	//glPushMatrix();
		apply();
	//glPopMatrix();
}