#pragma once

#include "Vector4f.h"
#include "Object.h"
#include "Color.h"

class Light: public Object {
private:
	// Light number
	GLuint _iLightNumber;				

	// Ambient, diffuse and position
	Color _ambient;
	Color _diffuse;
	Vector4f _position;

	//direction, cutoff, and exponent
	GLfloat _fDirection[3];
	GLfloat _fCutoff;
	GLfloat _fExponent;
	
	// setters
	void setAmbient(const Color &ambient);
	void setDiffuse(const Color &diffuse);
	void setPosition(const Vector4f &position);
	void setSpotlightVars();

public:
	Light();
	~Light();

	//common function to be used in construction
	void create(GLuint lightNum, const Color &ambient=Color::black(), const Color &diffuse=Color::white(), const Vector4f &position=Vector4f(0.0f,0.0f,0.0f,1.0f));

	//show the light
	void apply() const;

	//gets and sets for particular variable in position
	const float& operator[](int i) const;
	float& operator[](int i);

	//the read in function - receives the istream from the ObjectMeshManager
	void read(istream &fin);
	//the write function
	ostream& write(ostream &out);

	// update the light
	void update();
	
	//draw function from Object and called from ObjectMeshManager
	//calls apply();
	void draw() const;
};
