#include "Materials.h"

Materials::Materials() {
	_ambient = Color::black();
	_diffuse = Color::white();
}

/*------------------------------------
* common function used in construction
--------------------------------------*/
void Materials::create(const Color &ambient, const Color &diffuse, const Color &emission) {
	setAmbient(ambient);
	setDiffuse(diffuse);
	setEmission(emission);
}

/*-------------------------------------
* setters for the various color types
---------------------------------------*/
void Materials::setAmbient(const Color &ambient) {
	_ambient = ambient;
}		
void Materials::setDiffuse(const Color &diffuse) {
	_diffuse = diffuse;
}		
void Materials::setEmission(const Color &emission) {
	_emission = emission;
}		

/*-------------------------------------
* apply the material
---------------------------------------*/
void Materials::apply() const {
	glMaterialfv(GL_FRONT, GL_AMBIENT, _ambient.rgba());
	glMaterialfv(GL_FRONT, GL_DIFFUSE, _diffuse.rgba());
	glMaterialfv(GL_FRONT, GL_EMISSION, _emission.rgba());
}

/*--------------------------------------------
* fade down the diffuse color - used for blending
----------------------------------------------*/
void Materials::alphaDiffuse(const float inc) {
	_diffuse[3] += inc;
	if (_diffuse[3] < 0.0f) _diffuse[3] = 0.0f;
	if (_diffuse[3] > 1.0f) _diffuse[3] = 1.0f;
}

/*--------------------------------------------------
* set the alpha of the diffuse color to a specific value
----------------------------------------------------*/
void Materials::setAlpha(const float value) {
	if(value < 0.0f)
		_diffuse[3] = 0.0f;
	else if(value > 1.0f)
		_diffuse[3] = 1.0f;
	else
		_diffuse[3] = value;
}