#include "MySphere.h"

MySphere::~MySphere() {
	// delete the different arrays created with new[]
	delete [] _fVertexarray;
	delete [] _fNormalarray;
	delete [] _iIndexarray;
	delete [] _fTexarray;
}

/*---------------------------------------------------
* a create function that's used for creating a plane
* from other classes 
-----------------------------------------------------*/
void MySphere::create(const float radius, const GLuint texture){
	_fRadius = radius; // set the radius
	_iTextureNum = texture; // sets the textureNumber
	createArrays(); // create the arrays
	makeDisplayList(); // make the display list
}

/*---------------------------------------------------------
* create the sphere by creating a vertex array, normal array
* index array and texture coord array
------------------------------------------------------------*/
bool MySphere::createArrays() {
	//if stacks and slices are too low, return
	if ((_iStacks < 2) & (_iSlices <2)) return false;

	int iArray_size; //maximum size the vertex array etc. can be
	float fStack_inc; //stack incretement
	float fSlice_inc; //slice incretement
	float fX, fY , fZ; //used to find the position of points on the circle
	int iVertex_count; //current vertex
	int iIndex_count; //current index
	int iTemp_vc; //used to temp hold iVertex_count
	float fTemp_tex; //used to temp hold a tex value
	float fTemp_rad; //temp radius

	iArray_size = (_iSlices+1)*(_iStacks-1)+2;	// +2 is for the top and bottom vertex

	//create our arrays out to the proper sizes
	_fVertexarray = new float[iArray_size][3];
	_fNormalarray = new float[iArray_size][3];
	_iIndexarray = new GLuint[2+(_iStacks-1)*(_iSlices+1)*2];
	_fTexarray = new float[iArray_size][2];

	//work out how much to incretement by
	fStack_inc = 1.0f/(float)_iStacks;
	fSlice_inc = sPI*2.0f/_iSlices;

	/*----------------------------------------------
	* define the vertex array
	------------------------------------------------*/

	// top point
	iVertex_count = 0;
	_fVertexarray[iVertex_count][0] = 0.0f; //vertex array
	_fVertexarray[iVertex_count][1] = _fRadius; 
	_fVertexarray[iVertex_count][2] = 0.0f; 

	_fNormalarray[iVertex_count][0] = 0.0f; //normal array
	_fNormalarray[iVertex_count][1] = 1.0f; 
	_fNormalarray[iVertex_count][2] = 0.0f;

	_fTexarray[iVertex_count][0] = 0; // tex coord array
	_fTexarray[iVertex_count++][1] = 1; 

	// bottom point
	_fVertexarray[iVertex_count][0] = 0.0f; //vertex array
	_fVertexarray[iVertex_count][1] = -_fRadius; 
	_fVertexarray[iVertex_count][2] = 0.0f;

	_fNormalarray[iVertex_count][0] = 0.0f; //normal array
	_fNormalarray[iVertex_count][1] = -1.0f; 
	_fNormalarray[iVertex_count][2] = 0.0f;

	_fTexarray[iVertex_count][0] = 0; //tex coord array
	_fTexarray[iVertex_count++][1] = 0; 


	//run through the circle
	for (int i = 1; i < _iStacks; i++) {
		fY = sin(sPI*(1/2.0f - fStack_inc*(float)i)); //find the y value for this stack
		fTemp_rad = cos(sPI*(1/2.0f - fStack_inc*(float)i)); // the temp radius of the circle
															// - smaller near the top and bottom
		iTemp_vc = iVertex_count; // hold the current iVertex_count
		fTemp_tex = 1.0f - fStack_inc*(float)i; // hold the current tex coord value
		
		//run through the slices of the current stack
		for(int j = 0; j < _iSlices; j++) { 
			fX = cos((float)j*fSlice_inc); // find the x
			fZ = -sin((float)j*fSlice_inc);  // find the z

			_fVertexarray[iVertex_count][0] = _fRadius*fTemp_rad*fX; //vertex array
			_fVertexarray[iVertex_count][1] = _fRadius*fY;
			_fVertexarray[iVertex_count][2] = _fRadius*fTemp_rad*fZ;

			_fNormalarray[iVertex_count][0] = fTemp_rad*fX; //normal array
			_fNormalarray[iVertex_count][1] = fY;
			_fNormalarray[iVertex_count][2] = fTemp_rad*fZ;

			_fTexarray[iVertex_count][0] = (float)j/(float)_iSlices; //tex coord array
			_fTexarray[iVertex_count++][1] = fTemp_tex;
		};
		//the last vertex in each loop is the same as the first
		_fVertexarray[iVertex_count][0] = _fVertexarray[iTemp_vc][0]; //vertex array
		_fVertexarray[iVertex_count][1] = _fVertexarray[iTemp_vc][1];
		_fVertexarray[iVertex_count][2] = _fVertexarray[iTemp_vc][2];

		_fNormalarray[iVertex_count][0] = _fNormalarray[iTemp_vc][0]; //normal array
		_fNormalarray[iVertex_count][1] = _fNormalarray[iTemp_vc][1];
		_fNormalarray[iVertex_count][2] = _fNormalarray[iTemp_vc][2];

		_fTexarray[iVertex_count][0] = 1; // tex coord array
		_fTexarray[iVertex_count++][1] = fTemp_tex;
	};

	// now generate the index array
	// start with triangle fans for the top
	iIndex_count = 0;
	iVertex_count =2;
	_iIndexarray[iIndex_count++] = 0; // very top vertex
	for(int j = 0; j<= _iSlices; j++) {
		_iIndexarray[iIndex_count++] = iVertex_count++; //points around it
	};
	iVertex_count -= (_iSlices+1);

	// now do the main strips
	for(int i = 0; i< (_iStacks-2); i++) {
		for(int j = 0; j<= _iSlices; j++) {
			_iIndexarray[iIndex_count++] = iVertex_count++;
			_iIndexarray[iIndex_count++] = _iSlices+iVertex_count;
		};
	};

	_iIndexarray[iIndex_count++] = 1; // very bottom vertex
	for(int j = 0; j<= _iSlices; j++) {
		_iIndexarray[iIndex_count++] = iVertex_count+_iSlices-j; // points around it
	};

	return true;
}

/*-----------------------------------------------
* make the displayl list for the sphere
-------------------------------------------------*/
void MySphere::makeDisplayList(){
	// tell openGL that we're making a display list
	_dlSphere = glGenLists(1);

	// new compiled displaylist
	glNewList(_dlSphere, GL_COMPILE);

		glPushMatrix();
			glEnableClientState(GL_TEXTURE_COORD_ARRAY); //enable texture coord array and tell it where to find it
			glTexCoordPointer(2, GL_FLOAT, 0, _fTexarray);
			glEnableClientState(GL_NORMAL_ARRAY); // enable normal array and tell it where to find it
			glNormalPointer(GL_FLOAT, 0, _fNormalarray);
			glEnableClientState(GL_VERTEX_ARRAY); // enable vertex array and tell it where to find it
			glVertexPointer(3, GL_FLOAT, 0, _fVertexarray);
			
			//bind the texture we received
			glBindTexture(GL_TEXTURE_2D, _iTextureNum);

			//glDrawElements(What to draw, how many, type, address of where to start)
			//draw the top
			glDrawElements(GL_TRIANGLE_FAN, _iSlices+2, GL_UNSIGNED_INT, &_iIndexarray[0]);
			//draw the body of the sphere
			for (int i = 0; i < (_iStacks-2); i++) { 
				glDrawElements(GL_TRIANGLE_STRIP, (_iSlices+1)*2, GL_UNSIGNED_INT, &_iIndexarray[_iSlices+2+i*(_iSlices+1)*2]);
			};
			//draw the bottom
			glDrawElements(GL_TRIANGLE_FAN, _iSlices+2, GL_UNSIGNED_INT, &_iIndexarray[_iSlices+2+(_iStacks-2)*(_iSlices+1)*2]);
		glPopMatrix();

	//tell openGL that we're done with the displaylist
	glEndList();	
}

/*-----------------------------------------------------
* draw the sphere - calls the displaylist 
-------------------------------------------------------*/
void MySphere::draw() const {
	glCallList(_dlSphere);
}