#pragma once

#include "GXBase.h"
#include "TextureManager.h"
#include "Vector3d.h"
#include <string>
#include <iostream>

//primitives
#include "Cuboid.h"
#include "Cylinder.h"
#include "Plane.h"
#include "MySphere.h"

using namespace std;

class Object {
private:
	string _sTextureName;
	GLuint _iTextureNumber;
	Vector3d _position;
	
	string _sID;

public:
	Object();
	virtual ~Object(){};

	const string textureName() const { return _sTextureName; }; // get the texture name
	void textureName(const string tn){ _sTextureName = tn; }; // set the texture name

	const GLuint textureNumber() const{ return _iTextureNumber; }; // get the texture number
	GLuint textureNumber(const GLuint tnum); // set the texture number and return it

	GLuint getTexture(const string &name, bool isAlphaMask=false); // make a call to the TextureManager to
									 // return a GLuint for a texture

	// pure virtual functions will require derived classes to implement them
	virtual void read(istream &fin) = 0; // pure virtual read function
	virtual ostream& write(ostream &out) { return out; }; // virtual write function
	virtual void draw() const = 0; // pure virtual draw function

	virtual void update(){ return; } // update function used for the non-statics, eg bullets, bulletLines etc, to fade them over time
	virtual const bool isVisible() const { return true; } // return if it's visible - only used for non-statics

	virtual void drawBoundingBox() const { return; }; // draw the bounding box - used as it's called from the 
													  // ObjectMeshManager, which just holds pointers to Object's

	Vector3d& position() { return _position; }; // return by reference a Vector3d of the objects position
												// only returning by referencing so the user can read in 
												// and read out, and set direct to the position vector
	//return individual x, y, and z coords. can get this through the position()._x or similar, but 
	// this is quicker
	double xPos() const { return _position._x; }; // return the x position
	double yPos() const { return _position._y; }; // return the y position
	double zPos() const { return _position._z; }; // return the z position

	//setters for the position
	void position(const Vector3d &R); // set the object's positon using a Vector3d object
	void position(const double x, const double y, const double z); //set the objects position using doubles

	/*------------------------------*/
	void setID(const string& str) { _sID = str; };
	const string& getID() const { return _sID; };
	/*------------------------------*/
};

//streaming
inline istream &operator>>(istream &fin, Object &o) { o.read(fin); return fin; }