#include "ObjectMeshManager.h"

// define an iterator for the list of managed objects
typedef vector<Object*>::iterator oIter;
typedef vector<PhysicsObject*>::iterator opIter;
typedef vector<SecurityCamera*>::iterator scIter;

//initialize the instance to 0
ObjectMeshManager * ObjectMeshManager::_instance = 0;

/*-------------------------------------
* destructor
---------------------------------------*/
ObjectMeshManager::~ObjectMeshManager(){
	/*-------------------------------------------------------------
	* NOTE: we don't need to delete the _oAllPhyObjects list as
	* they'll be delted with the _oAllObjects list, so just set
	* the pointers to null. ditto with the security cameras
	---------------------------------------------------------------*/
	//run through the list of objects managed, and delete each one
	for (oIter i = _oAllObjects.begin(); i!=_oAllObjects.end(); i++) {
		delete (*i);
	}
	//non static objects
	for (oIter i = _oNonStatics.begin(); i!=_oNonStatics.end(); i++) {
		delete (*i);
	}
	//set the pointer on the _oAllPhyObjects list to null
	for (opIter i = _oAllPhyObjects.begin(); i!=_oAllPhyObjects.end(); i++) {
		*i = 0;
	}
	//set the pointer on the _oAllSecurityCameras list to null
	for (scIter i = _oAllSecurityCameras.begin(); i!=_oAllSecurityCameras.end(); i++) {
		*i = 0;
	}
}

/*------------------------------------------------------------
* the create function - create a basic Object and pass it the
* stream so it can read in it's own values
--------------------------------------------------------------*/
Object* ObjectMeshManager::create(const string &type, istream &fin) {
	//create a base object - we'll overwrite it shortly
	Object *object = 0;
	char c;

	if (type.compare("Stair")==0 || type.compare("Platform")==0 || type.compare("Pillar")==0 || type.compare("WallSection")==0 || type.compare("Monitor") == 0 || type.compare("Floor") == 0 || type.compare("Roof") == 0){
		object = new SolidBlockObject;
		fin >> c >> (*object) >> c; // new stair / platform / pillar / wallsection / monitor / floor / roof
		dynamic_cast< SolidBlockObject *>(object)->type(type); //set the type (do a dynamic cast first)
	} else if (type.compare("Skybox") == 0){
		object = new Skybox;
		fin >> c >> (*object) >> c; // new skybox
	}  else if (type.compare("Window") == 0){
		object = new Window;
		fin >> c >> (*object) >> c; // new window
	} else if (type.compare("Pipe") == 0){
		object = new Pipe;
		fin >> c >> (*object) >> c; // new pipe
	} else if (type.compare("Light") == 0){
		object = new Light;
		fin >> c >> (*object) >> c; // new light
	} else if (type.compare("SecurityCamera") == 0){
		object = new SecurityCamera;
		fin >> c >> (*object) >> c; // new security camera
	}
	else
		throw "File Parsing Error";
	return object;
}

/*------------------------------------------------
* go through the list of objects managed and get
* them to write themselves out to file
--------------------------------------------------*/
ostream& ObjectMeshManager::write(ostream &out){
	//run through the list of objects managed, and get them to write out their props
	for (oIter i = _oAllObjects.begin(); i!=_oAllObjects.end(); i++) {
		(*i)->write(out);
	}
	return out;
};

/*----------------------------------------------------
* go through the list of object managed and get them
* to draw themselves
------------------------------------------------------*/
void ObjectMeshManager::drawObjects(bool drawBoundingBox){
	//run through the list of objects managed, and draw them
	for (oIter i = _oAllObjects.begin(); i!=_oAllObjects.end(); i++) {
		(*i)->update();
		(*i)->draw();

		if(drawBoundingBox){
			glDisable(GL_LIGHTING); // temporaily disable lighting
			glDisable(GL_TEXTURE_2D); // disable texture2d
				//bounding boxes
				glColor3f(1.0f, 0.0f, 0.0f);
				glLineWidth(5.0f); //set a thick lineWidth
				(*i)->drawBoundingBox(); // draw the bounding box
				glLineWidth(1.0f); // reset the lineWidth
			glEnable(GL_TEXTURE_2D);
			glEnable(GL_LIGHTING); // reset lighting
		}
	}
	//draw the non-statics
	for (oIter i = _oNonStatics.begin(); i!=_oNonStatics.end();) { //don't ++ i here
		if((*i)->isVisible()){
			(*i)->update();
			(*i)->draw();
			i++; // no problem, interate through the list as normal
		} else {
			delete (*i); // delete the newed object
			i = _oNonStatics.erase(i); //erase the pointer from the vector list
									   // this returns the next one
		}
	}
}

/*-----------------------------------------
* add an object to the managed list
-------------------------------------------*/
void ObjectMeshManager::add(Object* obj){
	//add the object to the list of managed objects
	_oAllObjects.push_back(obj);
}

/*-----------------------------------------
* add a non-static object to the managed list
-------------------------------------------*/
void ObjectMeshManager::addNonStatic(Object* obj){
	//add the object to the list of managed objects
	_oNonStatics.push_back(obj);
}

/*------------------------------------------------------
* add a PhysicsObject object to the _oAllPhyObjects list
--------------------------------------------------------*/
void ObjectMeshManager::addPhysicsObj(PhysicsObject* pObj){
	//add the object to the list of managed objects
	_oAllPhyObjects.push_back(pObj);
}

/*------------------------------------------------------
* add a SecurityCamera object to the _oAllSecurityCameras list
--------------------------------------------------------*/
void ObjectMeshManager::addSecurityCameraObj(SecurityCamera* sObj){
	//add the object to the list of managed objects
	_oAllSecurityCameras.push_back(sObj);
	_iTotalSecurityCameras++;
}

/*-------------------------------------------------------------------
* get the next security camera in line - return 0 if there are none
---------------------------------------------------------------------*/
SecurityCamera * ObjectMeshManager::getNextSecurityCamera(){
	if(_iTotalSecurityCameras == 0) return 0; // if there is none return 0
	int temp = _iCurrSecurityCamera; // temp hold the current security camera
	_iCurrSecurityCamera = _iCurrSecurityCamera < _iTotalSecurityCameras-1 ? _iCurrSecurityCamera+1 : 0; //incretement the current one
	return _oAllSecurityCameras[temp]; //return the pointer
}