#pragma once

#include "GXBase.h"
#include "Object.h"
#include "PhysicsObject.h"
#include "Skybox.h"
#include "SolidBlockObject.h"
#include "Window.h" 
#include "Pipe.h"
#include "Light.h"
#include "SecurityCamera.h"

#include <vector>
#include <string>
#include <iostream>
using namespace std;

class ObjectMeshManager{
private:
	vector<Object*> _oAllObjects; // all the objects currently managed
	vector<Object*> _oNonStatics; // all the non-static object, such as bullets etc
	vector<PhysicsObject*> _oAllPhyObjects; // all the physics objects currently managed
	vector<SecurityCamera*> _oAllSecurityCameras; // all the security camera objects currently managed

	int _iTotalSecurityCameras;
	int _iCurrSecurityCamera;

	static ObjectMeshManager * _instance; // static instance pointer that will point to the only version of this

	//hide the gang of three to stop people making them
	ObjectMeshManager(): _iTotalSecurityCameras(0), _iCurrSecurityCamera(0) {};
	ObjectMeshManager(const ObjectMeshManager&){};
	ObjectMeshManager operator = (const ObjectMeshManager &) {};
	
public:
	~ObjectMeshManager();

	//return the _oAllObjects array - used in the PhysicsIF class
	vector<PhysicsObject*>& oAllPhyObjects(){ return _oAllPhyObjects; };

	//return or create an instance of the ObjectMeshManager - only one can exist
	static ObjectMeshManager * get(){
		if(_instance == 0){
			_instance = new ObjectMeshManager();
		}
		return _instance;
	};

	//free the instance resource. resets the pointer also so another may be created if wished
	// though only one will live at a time
	static void destroy(){
		if(_instance){
			delete _instance;
			_instance = 0;
		}
	}

	// create a stock object and pass the input stream to it, so it can
	// read it's own variable in
	static Object* create(const string &type, istream &fin);

	// call all the objects managed to write out their variables
	ostream& write(ostream &out);

	// call all the objects to draw themselves
	void drawObjects(bool drawBoundingBox);

	// add an object to the list
	void add(Object* obj);

	// add a non-static object to the list
	void addNonStatic(Object* obj);

	// add a PhysicsObject object to the _oAllPhyObjects list
	void addPhysicsObj(PhysicsObject* pObj);

	// add a SecurityCamera object to the _oAllSecurityCameras list
	void addSecurityCameraObj(SecurityCamera* sObj);

	// get the next security camera in line - return 0 if there are none
	SecurityCamera * getNextSecurityCamera();
};

// overloaded write operator
inline ostream &operator<<(ostream &out, ObjectMeshManager &omm) { omm.write(out); return out; }