#include "Particle.h"

/*--------------------------------------------------
* constructor - intialize the position, velocity,
* and life. age size and color are default for this
* but prob should be changed with the rest
----------------------------------------------------*/
Particle::Particle(const Vector3d &pos, const Vector3d &vel, const float life){
	_fLife = life; // set the life
	_color.create(Color::black(), Color(0.2f, 0.2f, 0.25f, 0.3f)); // create a default white color
	//call the create function to set the position, velocity, and age
	create(pos, vel);
}

/*-------------------------------------------------------------------------
* create function that used by the constructor and the reset function
---------------------------------------------------------------------------*/
void Particle::create(const Vector3d &pos, const Vector3d &vel){
	_position.set(pos); // set the position
	_velocity.set(vel); // set the velocity
	_fAge = 0.0f; // set the age and size
	_fSize = 0.5f;
	_bAlive = true;
}

/*------------------------------------------------------
* reset the particle to a new position and velocity
--------------------------------------------------------*/
void Particle::reset(const Vector3d & pos, const Vector3d & vel){
	create(pos, vel);
}

/*---------------------------------------------------------------------------
* update the particles position by adding it's velocity to it's prev position
* also update the age of the particle, and set it to dead if need be
-----------------------------------------------------------------------------*/
void Particle::update(){
	_position += _velocity*0.05;
	_fAge += 0.2f;
	if(_fAge > _fLife) _bAlive = false;
	_fSize += 0.01f;
	//change the alpha value based on life
	_color.setAlpha((_fLife - _fAge)/_fLife * 0.3f);
}

/*------------------------------------------------------------------
* the draw function - just draws the polygon, the texture is applied from
* the ParticleEngine
--------------------------------------------------------------------*/
void Particle::draw() const{
	_color.apply(); // apply the color

	float matrix[16]; // define a 4x4 matrix to hold the current one

	glPushMatrix();
		glTranslated(_position._x, _position._y, _position._z); // translate into position

		//load the current matrix and change the rotation so the particles are billboarded
		glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
		matrix[0] = matrix[5] = matrix[10] = 1.0f;
		matrix[1] = matrix[2] = matrix[4] = matrix[6] = matrix[8] = matrix[9] = 0.0f;
		glLoadMatrixf(matrix);
		
		//draw the quad
		glBegin(GL_QUADS);
			glNormal3f(0.0f, 0.0f, 1.0f);
			glTexCoord2d(0.0, 0.0);		glVertex3d(-_fSize, - _fSize, 0.0);
			glTexCoord2d(1.0, 0.0);		glVertex3d( _fSize, - _fSize, 0.0);
			glTexCoord2d(1.0, 1.0);		glVertex3d( _fSize,   _fSize, 0.0);
			glTexCoord2d(0.0, 1.0);		glVertex3d(-_fSize,   _fSize, 0.0);
		glEnd();
	glPopMatrix();
}