#pragma once

#include "Vector3d.h"
#include "Materials.h"

class Particle {
private:
	Vector3d _position, _velocity; // vectors to hold the position and velocity of the particle
	Materials _color; //color, fade etc
	float _fAge, _fLife, _fSize; // the age of the particle, it's lifespan, and size
	bool _bAlive; // is the particle alive?

	//create function that used by the constructor and the reset function
	void create(const Vector3d & pos, const Vector3d & vel);

public:
	//constructor - intialize the position, velocity, and life. 
	//age, size and color are default for this but prob should be changed with the rest
	Particle(const Vector3d & pos, const Vector3d & vel, const float life);

	//the draw function - just draws the polygon, the texture is applied from
	// the ParticleEngine
	void draw() const;

	//update the particles position by adding it's velocity to it's prev position
	//also update the age of the particle, and set it to dead if need be
	void update();

	//reset the particle to a new position and velocity
	void reset(const Vector3d & pos, const Vector3d & vel);

	//see is the particle alive or not
	const bool isAlive() const { return _bAlive; };

	//set the particles aliveness
	void setAlive(const bool b) { _bAlive = b; };
};