#pragma once

#include "Object.h"
#include "Particle.h"
#include <vector>

class ParticleEngine: public Object {
private:
	int _iMaxParticles; // maximum particles allowed
	int _iBatchRelease; // how many particles to release at a time
	int _iReleased; // number of particles released
	vector<Particle*> _particles; // array to hold the particles created
	Vector3d _direction; // direction the particle engine is facing
	GLuint _texNum; // textureNumber
	
	void * _particleMemory; //void pointer so we can grab a ball of memory
	Particle * _memory; // Particle pointer so we use _particleMemory and incretement it

	//set the particle velocity - common function used by ParticleEngine() and resetParticles()
	Vector3d setParticleVelocity(){
		return Vector3d(_direction._x + getRandomNumber(),
						_direction._y + getRandomNumber(),
						_direction._z + getRandomNumber());
	};

public:
	//constructor - intialize the position and direction. other variable are presets
	ParticleEngine(const Vector3d & pos, const Vector3d & dir, const int numParticles);

	//destructor - delete all the particles created
	~ParticleEngine();

	//get a random number between -0.2 and 0.2
	const float getRandomNumber(){ return -0.2f + (float)rand()/(float)RAND_MAX * 0.4f; };

	//update the particle engine - release more particles, revive dead ones
	void update();

	//release a set number of particles
	void releaseParticles();

	//reset the particle to a default position and set it off again
	void resetParticle(Particle * ptParticle);

	//move the particle engine to a new position and normal
	void move(const Vector3d & pos, const Vector3d & dir);

	//the draw function - applies the texture to the particle and calls the draw function
	// for all of them
	void draw() const;

	//the read function - not used for ParticleEngine, but needed to instanstiate
	void read(istream &){ return; };
};