#include "ParticleManager.h"

// define an iterator for the list of managed objects
typedef vector<ParticleEngine*>::iterator peIter;

#include <fstream>

//initialize the instance to 0
ParticleManager * ParticleManager::_instance = 0;

/*-------------------------------------
* destructor
---------------------------------------*/
ParticleManager::~ParticleManager(){
	//run through the list of objects managed, and delete each one
	for (peIter i = _pEngines.begin(); i!=_pEngines.end(); i++)
		delete (*i);
}

/*------------------------
* read from a stream
--------------------------*/
void ParticleManager::read(std::istream &fin){
	char buffer[256], ch;
	
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _iMaxEngines; // read in the _iMaxEngines

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _iParticlesPerEngine; // read in the _iParticlesPerEngine
}

/*--------------------------
* write to a stream
----------------------------*/
ostream& ParticleManager::write(ostream &out){
	out << "**ParticleManager**"
		 << "\n\tmaxParticleEngines= " << _iMaxEngines
		 << "\n\tmaxParticlesPerEngine= " << _iParticlesPerEngine << endl;
	return out;
}

/*----------------------------------------------------------------------------------
* a request to add a particle engine. if we haven't reached our max amount, then
* create a new one, else move an existing one. receives the position to create
* the particle engine, as well as the normal of the surface
------------------------------------------------------------------------------------*/
void ParticleManager::addParticleEngine(const Vector3d &pos, const Vector3d &normal){
	if(_iEnginesCreated < _iMaxEngines){ // haven't reached max number yet
		ParticleEngine * pe = new ParticleEngine(pos, normal, _iParticlesPerEngine);
		_pEngines.push_back(pe);
		_iEnginesCreated++; // incretement the current engine
	} else {
		_pEngines[_iCurrEngine]->move(pos, normal); // move one of the previous particleEngines
		_iCurrEngine = _iCurrEngine < _iMaxEngines-1 ? _iCurrEngine + 1 : 0;
	}
}

/*----------------------------------------------------
* go through the list of object managed and get them
* to draw themselves
------------------------------------------------------*/
void ParticleManager::drawParticleEngines(){
	//run through the list of objects managed, and draw them
	for (peIter i = _pEngines.begin(); i!=_pEngines.end(); i++){
		(*i)->update();
		(*i)->draw();
	}
}