#pragma once

#include "ParticleEngine.h"
#include "Particle.h"

#include <vector>
#include <new>
#include "Vector3d.h"

class ParticleManager{
private:
	vector<ParticleEngine*> _pEngines; // all the particle engines currently managed
	int _iMaxEngines, _iParticlesPerEngine; // max engines allowed, particle per engine
	int _iEnginesCreated; // current number of engines created
	int _iCurrEngine; // current particle engine
	string _sTexture; // texture for the particle

	static ParticleManager * _instance; // static instance pointer that will point to the only version of this

	//hide the gang of three to stop people making them
	ParticleManager():_iMaxEngines(10), _iParticlesPerEngine(100), _iCurrEngine(0), _iEnginesCreated(0){};
	ParticleManager(const ParticleManager&){};
	ParticleManager operator = (const ParticleManager &) {};

	~ParticleManager();
	
public:
	//return or create an instance of the ParticleManager - only one can exist
	static ParticleManager * get(){
		if(_instance == 0){
			_instance = new ParticleManager();
		}
		return _instance;
	};

	//free the instance resource. resets the pointer also so another may be created if wished
	// though only one will live at a time
	static void destroy(){
		if(_instance){
			delete _instance;
			_instance = 0;
		}
	}

	//read in from a stream
	void read(istream &fin);

	//write out the different vars
	ostream& write(ostream &out);

	//a request to add a particle engine. if we haven't reached our max amount, then
	//create a new one, else move an existing one. receives the position to create
	// the particle engine, as well as the normal of the surface
	void addParticleEngine(const Vector3d & pos, const Vector3d & normal);

	// call all the objects to draw themselves
	void drawParticleEngines();
};

//streaming
inline istream &operator>>(istream &fin, ParticleManager * pm) { pm->read(fin); return fin; }
inline ostream &operator<<(ostream &out, ParticleManager * pm) { pm->write(out); return out; }