#pragma once

#include "GXBase.h"
#include "Vector3d.h"
#include <vector>

#define PI 3.141592
#define NEAR_ZERO 1.0E-8

class Player; // forward declaration of player
class Gun; // forward declaration of gun
class PhysicsObject; // forward declaration of physicsObject

class PhysicsIF {
private:
	//check to see if a Vector3d point is inside or on an objects bounding box
	bool checkPointInsideBox(const PhysicsObject * envObj, const Vector3d& point);

public:
	// convert angles to radians
	double angleToRadian(const double angle) const { return (angle * PI) / 180.0; };

	// convert radians to angles
	double radianToAngle(const double radian) const { return (radian * 180.0) / PI; };

	// get the next true move for the player - ie change it if he collides with something
	// work off a 'future' Vector3d position - ie, where the player will be
	// the isGravityChecking bool is there for when the players position is
	// being checked as a result of gravity rather than movement, it's so we
	// don't sink into objects if we fall on them
	Vector3d getNextTrueMove(const Player * ptPlayer, const Vector3d & nextEyePos, const bool isGravityChecking);

	//check to see if the bullet that the player fired hit anything. bullet's start pos
	// is the gun's position, direction is the gun's _bullet vector. use plane-line
	// collision to see if the player hit anything
	bool checkBulletCollision(const Gun * ptGun, const Vector3d & gunPos, const Vector3d & bulletDir, Vector3d & hitPos, Vector3d & hitNormal);

	//check the bullet against a cylinder. uses a different equation (courtesy of http://nehe.gamedev.net/data/lessons/lesson.asp?lesson=30)
	// so it's sent to a seperate function
	bool checkBulletCylinder(PhysicsObject * envObj, const Vector3d & gunPos, const Vector3d & bulletDir, double & lamda, Vector3d & hitPos, Vector3d & hitNormal);

};