#pragma once

#include "Object.h"
#include "PhysicsIF.h"
#include "Vector3d.h"
#include "BoundingBox.h"
#include "BulletHole.h"

class PhysicsObject: public Object, public PhysicsIF {
private:
	float _fWidth, _fHeight, _fDepth; // width, height, depth of the object
	BoundingBox _boundingBox;
	
public:
	virtual ~PhysicsObject(){}; // virtual destructor

	// and onShot function for the object to do stuff
	// first Vector3d is the hitPosition, the second is the hitNormal
	virtual void onShot(Vector3d &, Vector3d &){ return; };

	void drawBoundingBox() const; // draw it
	void setBoundingBox(); // set it up to its default
	void setBoundingBox(const float w, const float h, const float d, const Vector3d& pos); // set the bounding box up with params - position is set using Vector3d
	void setBoundingBox(const float w, const float h, const float d, const double x, const double y, const double z); // set the bounding box up with params - position is set using floats
	void moveBoundingBox(const double x, const double y, const double z); // move the bounding box
	void moveBoundingBox(const Vector3d &pos); // move the bounding box using a Vector3d

	//get the different bounds of the box
	const double getMinusXBound() const { return _boundingBox.getMinusXBound(); };
	const double getMinusYBound() const { return _boundingBox.getMinusYBound(); };
	const double getMinusZBound() const { return _boundingBox.getMinusZBound(); };
	const double getPlusXBound()  const { return _boundingBox.getPlusXBound();  };
	const double getPlusYBound()  const { return _boundingBox.getPlusYBound();  };
	const double getPlusZBound()  const { return _boundingBox.getPlusZBound();  };

	//get the different points on the bounding box faces
	const Vector3d* getMinusXPoint() const { return _boundingBox.getMinusXPoint(); };
	const Vector3d* getMinusYPoint() const { return _boundingBox.getMinusYPoint(); };
	const Vector3d* getMinusZPoint() const { return _boundingBox.getMinusZPoint(); };
	const Vector3d* getPlusXPoint()  const { return _boundingBox.getPlusXPoint();  };
	const Vector3d* getPlusYPoint()  const { return _boundingBox.getPlusYPoint();  };
	const Vector3d* getPlusZPoint()  const { return _boundingBox.getPlusZPoint();  };

	//get the different normals on the bounding box faces
	const Vector3d* getMinusXNormal() const { return _boundingBox.getMinusXNormal(); };
	const Vector3d* getMinusYNormal() const { return _boundingBox.getMinusYNormal(); };
	const Vector3d* getMinusZNormal() const { return _boundingBox.getMinusZNormal(); };
	const Vector3d* getPlusXNormal()  const { return _boundingBox.getPlusXNormal();  };
	const Vector3d* getPlusYNormal()  const { return _boundingBox.getPlusYNormal();  };
	const Vector3d* getPlusZNormal()  const { return _boundingBox.getPlusZNormal();  };

	//see if the object is broken or not - can be used by the window
	virtual bool isBroken() const { return false; };

	float fWidth() const { return _fWidth; }; // get the width
	float fHeight() const{ return _fHeight;}; // get the height
	float fDepth() const { return _fDepth; }; // get the depth

	void fWidth(const float w) { _fWidth = w; } //set the width
	void fHeight(const float h){ _fHeight = h;} // set the height
	void fDepth(const float d) { _fDepth = d; } // set the depth
};