#include "Plane.h"

/*----------------------------------------------------
* constructor - set the variables that won't be set if
* the plane is created from another class, such as
* Cuboid
------------------------------------------------------*/
Plane::Plane(){
	_divisor = 10;
};

/*--------------------------------------------------
* destructor - delete the vertex, normal, index and
* tex arrays
----------------------------------------------------*/
Plane::~Plane(){
	delete [] _fVertexarray;
	delete [] _fNormalarray;
	delete [] _iIndexarray;
	delete [] _fTexarray;
}

/*---------------------------------------------------
* a create function that's used for creating a plane
* from other classes such as Cuboid
-----------------------------------------------------*/
void Plane::create(const float w, const float h, const GLuint iTextureNnum, const bool repeatTexture){
	_fWidth = w; // set the width
	_fHeight = h; // set the height
	_iTextureNum = iTextureNnum; // sets the textureNumber
	_textureDiv = repeatTexture ? 4.0f : 1.0f;
	createArrays(w, h, _divisor, _divisor);
	makeDisplayList(); // make the display list
}

/*-----------------------------------------------------------
* the createArrays function defines the vertex, normal, index
* and tex arrays for the plane
*
* TODO - make the number of rows and columns streamable
-------------------------------------------------------------*/
bool Plane::createArrays(const float width, const float height, const int rows, const int columns) {
	//if stacks and slices are too low, return
	if ((rows < 1) & (columns < 1)) return false;

	int iArray_size; //maximum size the vertex array etc. can be
	float f_xinc = width/columns; //row incretement
	float f_yinc = height/rows; //column incretement
	float fTlcx = -width/2; //top left hand corner x
	float fTlcy = height/2; //top left hand corner y
	float fX, fY; //used to find the position of points on the plane
	int iVertex_count = 0; //current vertex
	int iIndex_count = 0; //current index

	_iRows = rows;
	_iColumns = columns;

	iArray_size = (rows+1)*(columns+1); // find out the size of our vertex, normal, and tex arrays

	//create our arrays out to the proper sizes
	_fVertexarray = new float[iArray_size][3];
	_fNormalarray = new float[iArray_size][3];
	_iIndexarray = new GLuint[(rows*(columns+1))*2];
	_fTexarray = new float[iArray_size][2];

	/*----------------------------------------------
	* define the vertex array
	------------------------------------------------*/

	//run through the plane
	for(int i=0; i<=rows; i++){
		for(int j=0; j<=columns; j++){
			fX = fTlcx + f_xinc*j; // starting x point
			fY = fTlcy - f_yinc*i; // starting y point

			_fVertexarray[iVertex_count][0] = fX; //vertex array
			_fVertexarray[iVertex_count][1] = fY;
			_fVertexarray[iVertex_count][2] = 0.0f;

			_fNormalarray[iVertex_count][0] = 0.0f; //normal array
			_fNormalarray[iVertex_count][1] = 0.0f;
			_fNormalarray[iVertex_count][2] = 1.0f;

			_fTexarray[iVertex_count][0] = (1.0f/columns)*j * _textureDiv; //tex coord array
			_fTexarray[iVertex_count][1] = 1 - (float)i/(float)rows * _textureDiv;
			
			iVertex_count++;
		};
	};

	// now generate the index array
	iIndex_count = 0;
	iVertex_count = 0;

	// plane
	for(int i = 0; i< columns+1; i++) {
		for(int j = 0; j< rows; j++) {
			_iIndexarray[iIndex_count++] = iVertex_count++;
			_iIndexarray[iIndex_count++] = columns + iVertex_count;
		};
	};

	return true;
}

/*-----------------------------------------------
* make the displayl list for the plane
-------------------------------------------------*/
void Plane::makeDisplayList(){
	// tell openGL that we're making a display list
	_dlPlane = glGenLists(1);

	// new compiled displaylist
	glNewList(_dlPlane, GL_COMPILE);

		glEnableClientState(GL_TEXTURE_COORD_ARRAY); //enable texture coord array and tell it where to find it
		glTexCoordPointer(2, GL_FLOAT, 0, _fTexarray);
		glEnableClientState(GL_NORMAL_ARRAY); // enable normal array and tell it where to find it
		glNormalPointer(GL_FLOAT, 0, _fNormalarray);
		glEnableClientState(GL_VERTEX_ARRAY); // enable vertex array and tell it where to find it
		glVertexPointer(3, GL_FLOAT, 0, _fVertexarray);
		
		//bind the relevant texture to the plane
		glBindTexture(GL_TEXTURE_2D, _iTextureNum);
		
		glPushMatrix();
			//draw the plane
			for (int i = 0; i < _iRows; i++) {
				// glDrawElements(what to draw, how many points, type, address of the first index)
				glDrawElements(GL_TRIANGLE_STRIP, (_iColumns+1)*2, GL_UNSIGNED_INT, &_iIndexarray[(_iColumns+_iRows+2)*i]);
			};
		glPopMatrix();

	//tell openGL that we're done with the displaylist
	glEndList();	
}

/*-------------------------------------
* the draw function - draw the vertex array
---------------------------------------*/
void Plane::draw() const{
	glCallList(_dlPlane);
}