#include "SecurityCamera.h"

/*-------------------------------------------------------------------------------------
* init the security camnera with a camera and pos - pos, fov, aspect, and farPlane all come from the View class
---------------------------------------------------------------------------------------*/
void SecurityCamera::create(const Vector3d& pos, const float fov, const float aspect, const float farPlane){
	position().set(pos);
	setupCamera(fov, aspect, farPlane);
}

/*-------------------------------------------------------------------------------------
* init the security camera with a camera and pos - x, y, z, fov, aspect, and farPlane all come from the View class
---------------------------------------------------------------------------------------*/
void SecurityCamera::create(const float x, const float y, const float z, const float fov, const float aspect, const float farPlane){
	position().set(x, y, z);
	setupCamera(fov, aspect, farPlane);
}

/*----------------------------------
* set up the security camera's camera
------------------------------------*/
void SecurityCamera::setupCamera(const float fov, const float aspect, const float farPlane){
	_camera.setCameraShape(fov, aspect, 0.1, farPlane);
	_camera.setCameraPos(position());
	capture(); // capture straight away to set up the texture
}

/*------------------------
* show the view from the players eye(s)
--------------------------*/
void SecurityCamera::showView(){
	_camera.setMatrix();
}

/*-------------------------------
* look at a specific point
---------------------------------*/
void SecurityCamera::lookAt(const Vector3d & pos){
	_camera.setCameraLookAt(pos);
}

/*-------------------------------------------------
* capture the current viewport into a texture image
---------------------------------------------------*/
void SecurityCamera::capture(){
	_image.GrabGLViewport(); // grab it
	for(int i = 0; i < _image.Width(); i++){
		for(int j = 0; j < _image.Height(); j++){
			float ran = (float)rand()/(float)RAND_MAX;
			if(ran < _fInterference){
				float r, g, b;
				_image.GetPixel3f(i, j, r, g, b);
				r += ran * 255.0f;
				g += ran * 255.0f; 
				b += ran * 255.0f;
				_image.SetPixel3f(i, j, r, g, b);
			}
		}
	}
	TextureManager::get()->screenToTexture(_image, getID()); // render to a texture
}

/*-------------------------------------
* the draw function - draw the sphere
---------------------------------------*/
void SecurityCamera::draw() const{
	glPushMatrix();
		glTranslated(xPos(), yPos(), zPos());
		_sphere.draw();
	glPopMatrix();
}

/*----------------------
* the read function
------------------------*/
void SecurityCamera::read(istream &fin){
	char buffer[256], ch;
	string temp;

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fFov; // read in the fov

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fAspectRatioX; // read in the aspectRatioX

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fAspectRatioY; // read in the aspectRatioY
	
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fFarPlane; // read in the farPlane

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> position(); // read in the position

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _textureSize; // read in the texture size to render to

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> _fInterference; // read in the _quality
	if(_fInterference < 0.0f) _fInterference = 0.0f;
	if(_fInterference > 1.0f) _fInterference = 1.0f;

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> ch >> temp >> ch; // read in the texture
	textureName(temp); // set the texture name
	_texNum = getTexture(temp); // get the number

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> temp; // read in the id
	setID(temp);

	//create the sphere
	_sphere.create(0.3f, _texNum);

	//set up the camera
	setupCamera(_fFov, _fAspectRatioX/_fAspectRatioY, _fFarPlane);
}

/*---------------------------------------------
* the write function - stream out the data
-----------------------------------------------*/
ostream& SecurityCamera::write(ostream &out){
	out << "SecurityCamera {"
		 << "\n\tfov= " << _fFov
		 << "\n\taspectRatioX= " << _fAspectRatioX
		 << "\n\taspectRatioY= " << _fAspectRatioY
		 << "\n\tfarPlane= " << _fFarPlane
		 << "\n\tposition= " << position()
		 << "\n\ttexSize= " << _textureSize
		 << "\n\tinterference= " << _fInterference
		 << "\n\ttexture= " << textureName()
		 << "\n\tid= " << getID()
		 << "\n}";
	return out;
}