#pragma once

#include "Object.h"
#include "Camera.h"

class SecurityCamera : public Object{
private:
	int _textureSize; // rendered image texture size (width x height)
	float _fInterference; // the quality of the camera
	float _fFov, _fAspectRatioX, _fAspectRatioY, _fFarPlane; // fov, aspect ratio, and far plane for the camera
	Camera _camera; // camera used for displaying the scene
	Image _image; // Image used to capture the camera data etc
	MySphere _sphere; // physical representation of the camera
	GLuint _texNum; // texture number

	void setupCamera(const float fov, const float aspect, const float farPlane); // set up the security camera's camera

public:
	//init the security camera with a camera and pos - pos, fov, aspect, and farPlane all come from the View class
	// used when creating a security camera directly, not from streaming from a file
	void create(const Vector3d & pos, const float fov, const float aspect, const float farPlane);

	//init the security camera with a camera and pos - x, y, z, fov, aspect, and farPlane all come from the View class
	// used when creating a security camera directly, not from streaming from a file
	void create(const float x, const float y, const float z, const float fov, const float aspect, const float farPlane);

	//show the view from the security camera eye
	void showView();

	//look at a specific point
	void lookAt(const Vector3d & pos);

	//capture the current viewport into a texture image
	void capture();

	int getTexSize() const { return _textureSize; };

	//the draw function
	void draw() const;

	//the read function
	void read(istream &fin);
	
	//the write function
	ostream& write(ostream &out);
};

// overloaded write operator
inline ostream &operator<<(ostream &out, SecurityCamera &sCamera) { sCamera.write(out); return out; }