#include "SolidBlockObject.h"
#include "ObjectMeshManager.h"

class Object;

/*-----------------------------------------
* onShot - draw the bullethole
-------------------------------------------*/
void SolidBlockObject::onShot(Vector3d &hitPos, Vector3d &hitNormal){
	Object * bh = new BulletHole(hitPos, hitNormal); // new bullet hole
	ObjectMeshManager::get()->addNonStatic(bh);
}

/*-------------------------------------
* the draw function - calls on the
* object to draw
---------------------------------------*/
void SolidBlockObject::draw() const{
	glPushMatrix();
		glTranslated(xPos(), yPos(), zPos());
		if(_fRotateAngle != 0) glRotatef(_fRotateAngle, _fRotateX, _fRotateY, _fRotateZ);
		_SolidBlockObject.draw();
	glPopMatrix();
}

/*------------------------------------------
* sets the type of the SolidBlockObject.
* we need this for the streaming out, as this
* object stands in for stair, pillar, and platform.
* this function is called from ObjectMeshManager
--------------------------------------------------*/
void SolidBlockObject::type(const string &type){
	_type = type;
}

/*-------------------------------------------------------------------------
* the read function - gets passed the istream from the ObjectMeshManager
---------------------------------------------------------------------------*/
void SolidBlockObject::read(istream &fin){
	char buffer[256], ch;
	float w, h, d; // width, height, and depth
	
	// create two array, one for the texture strings for each side
	// and one for the corresponding number variables in this object
	// (need to make arrays of pointers to the variables themselves)
	string * sStreamTexNames[] = {&_sFrontTexture, &_sBackTexture, &_sLeftTexture, &_sRightTexture, &_sTopTexture, &_sBottomTexture};
	GLuint * iTexNumArray[] = {&_iFrontTexture, &_iBackTexture, &_iLeftTexture, &_iRightTexture, &_iTopTexture, &_iBottomTexture};

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> w; // read in the width
	fWidth(w); //set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> h; // read in the height
	fHeight(h); // set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> d; // read in the depth
	fDepth(d); // set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> position(); // read in the position

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> ch >> _fRotateAngle >> ch >> _fRotateX >> ch >> _fRotateY >> ch >> _fRotateZ >> ch; // read in the rotation

	for(int i=0; i<6; i++){
		fin.get(buffer, sizeof(buffer), '=');
		fin >> ch >> ch >> *sStreamTexNames[i] >> ch; // read in the six textures
		*iTexNumArray[i] = getTexture(*sStreamTexNames[i]); // assign the texture number
	}

	/*--------------------------------------------------------*/
	string tempStr;
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> ch >> tempStr >> ch; // read in the id
	setID(tempStr); // set it
	/*--------------------------------------------------------*/

	bool repeatTex = true;
	if(tempStr == "monitor") repeatTex = false;

	_SolidBlockObject.create(fWidth(), fHeight(), fDepth(),
				  _iFrontTexture, _iBackTexture, _iLeftTexture, _iRightTexture, _iTopTexture, _iBottomTexture,
				  repeatTex); // create the cuboid

	setupBoundingBox(); // create the bounding box
}

/*---------------------------------------------
* the write function - stream out the data
-----------------------------------------------*/
ostream& SolidBlockObject::write(ostream &out){
	out << _type << " {"
		 << "\n\twidth= " << fWidth()
		 << "\n\theight= " << fHeight()
		 << "\n\tdepth= " << fDepth()
		 << "\n\tposition= " << position()
		 << "\n\trotation= (" << _fRotateAngle << ", " << _fRotateX << ", " << _fRotateY << ", " << _fRotateZ << ")" 
		 << "\n\tfrontTexture= \"" << _sFrontTexture << " \""
		 << "\n\tbackTexture= \"" << _sBackTexture << " \""
		 << "\n\tleftTexture= \"" << _sLeftTexture << " \""
		 << "\n\trightTexture= \"" << _sRightTexture << " \""
		 << "\n\ttopTexture= \"" << _sTopTexture << " \""
		 << "\n\tbottomTexture= \"" << _sBottomTexture << " \""
		 << "\n\tid= \"" << getID() << " \""
		 << "\n}" << endl;
	return out;
}

/*---------------------------------------------
* set the bounding box - calls Object::setBoundingBox()
-----------------------------------------------*/
void SolidBlockObject::setupBoundingBox(){
	double tempw = xPos() + fWidth() * 0.5;
	double temph = yPos() + fHeight() * 0.5;
	double tempd = zPos() + fDepth() * 0.5;
	setBoundingBox(fWidth(), fHeight(), fDepth(), tempw, temph, tempd);
}