#pragma once

#include "GXBase.h"
#include <map>
#include <string>
#include <iostream>
using namespace std;
using namespace gxbase;

// the map contains a pair - the left element is the (string) texture name loaded
// the right element is the (GLuint) texture number
typedef pair<const string, const GLuint> Pair;
typedef map<const string, const GLuint> Map;
typedef map<const string, const GLuint>::iterator mIter;

class TextureManager{
private:
	// collection of textures
	Map _texturesLoaded;
	
	//GLuint _iCurrTexture; // the number of the current texture we're on
	Image _image; // an image object to temp hold the image loaded in

	//search the _sTextureNamesLoaded list to see if a texture has already been loaded or not
	int searchLoaded(const string &name);

	//create a texture
	GLuint create(const string &name);

	//create an alpha mask
	GLuint createAlphaMask(const string &name);

	//hold a static instance of this class
	static TextureManager * _instance;

	//hide the gang of three to stop people making them
	TextureManager(){};
	TextureManager(const TextureManager&){};
	TextureManager operator = (const TextureManager &) {};
	
public:
	//delete the pairs created
	~TextureManager();

	//return or create an instance of the TextureManager - only one can exist
	static TextureManager * get(){
		if(_instance == 0){
			_instance = new TextureManager();
		}
		return _instance;
	};

	//free the instance resource. resets the pointer also so another may be created if wished
	// though only one will live at a time
	static void destroy(){
		if(_instance){
			delete _instance;
			_instance = 0;
		}
	}

	//retrieve the number of a created texture, or create one if not there
	GLuint getTexture(const string &name, bool isAlphaMask=false);

	//save the screen to a texture - provide the name of the object that's doing the saving.
	// if the object has created a texture before, then find the number and overwrite it,
	// otherwise create one and save it
	void screenToTexture(Image& image, const string &name);
};