#include "View.h"

//constructor
View::View():_bGourand(0) {};

//destructor - destroy the ObjectMeshManager
View::~View(){
	ObjectMeshManager::destroy();
	TextureManager::destroy();
	ParticleManager::destroy();
}

/*------------------------------------------------------------
* PARSE the data for the View class from the environment.txt
* text file
-------------------------------------------------------------- */
bool View::parseData(){

	//open the environment.txt file to read in the data
	ifstream fin("environment.txt");
	if (!fin) {
		cerr << "Error: Failed to open environment file for reading" << endl;
		return false;
	}

	//temp vars for streaming
	string sAttribute;
	char c;

	while (fin >> sAttribute, sAttribute[0] != '}') {
		if(sAttribute.compare("setSize") == 0)
		{
			//setSize(int, int)
			fin >> c >> _iWidth >> c >> _iHeight >> c;
		}
		else if(sAttribute.compare("setDepthBits") == 0)
		{
			//setDepthBits(int)
			fin >> c >> _iDepthBits >> c;
		}
		else if(sAttribute.compare("setPosition") == 0)
		{
			//setPosition(int)
			fin >> c >> _iX >> c >> _iY >> c;
		}
		else if(sAttribute.compare("fullscreen") == 0)
		{
			//fullscreen(bool)
			fin >> c >> _bFullscreen >> c;
		}
		else if(sAttribute.compare("wireframe") == 0)
		{
			//wireframe(bool)
			fin >> c >> _bWireframe >> c;
		}
		else if(sAttribute.compare("background") == 0)
		{
			//background(float, float, float, float) - between 0 and 1
			float r, g, b, a;
			fin >> c >> r >> c >> g >> c >> b >> c >> a >> c;
			_bgColor = Color(r, g, b, a);
		}
		else if(sAttribute.compare("**ParticleManager**") == 0)
		{
			fin >> ParticleManager::get(); // read in the particle attributes
		}

	}

	//read in the player
	while(fin >> sAttribute, sAttribute[0] != '}'){
		fin >> _player;
	}

	//a bool to determine whether we're currently in a comment or not
	bool comment = 0;

	//end of scene object, start of primitives
	while (fin >> sAttribute) {
		//if the sAttribute == "#", and we're not in a comment, then its the start of
		//one. if we are, then its the end
		if(sAttribute[0] == '#'){
			comment = !comment; // set the flag
			//if(!comment) fin >> sAttribute; // read in the last #
			continue; // added instead of the previous line to allow for consecutive comments
		}
		// get the ObjectMeshManager to create a base object, and pass it the stream
		// so it can read it's own values in
		if(!comment){
			Object* obj = ObjectMeshManager::get()->create(sAttribute, fin);
			ObjectMeshManager::get()->add(obj); // add the object to the ObjectMeshManager's list

			PhysicsObject* pEnvObj = dynamic_cast< PhysicsObject *>(obj); // use a dynamic cast to see if it's a PhysicsObject
			if(pEnvObj) ObjectMeshManager::get()->addPhysicsObj(pEnvObj); // add it to the physics list, if it's a PhysicsObject

			SecurityCamera* scEnvObj = dynamic_cast< SecurityCamera *>(obj); // use a dynamic cast to see if it's a SecurityCamera
			if(scEnvObj) ObjectMeshManager::get()->addSecurityCameraObj(scEnvObj); // add it to the security camera list, if it's a SecurityCamera
		}
	}

	return true;
}

/*------------------------------------------------------------
* WRITE the data for the View class to the log
* text file
-------------------------------------------------------------- */
void View::writeData(){

	//open a stream to write the data out to
	ofstream fout("log.txt");
	if (!fout) {
		cerr << "Error: Failed to open log file for writing" << endl;
		return;
	}

	//stream out the scene object first, then the rest
	fout << "Scene{"
		 << "\n\tsetSize (" << _iWidth << ", " << _iHeight << ")"
		 << "\n\tsetDepthBits (" << _iDepthBits << ")"
		 << "\n\tsetPosition (" << _iX << ", " << _iY << ")"
		 << "\n\tfullscreen (" << _bFullscreen << ")"
		 << "\n\twireframe (" << _bWireframe << ")"
		 << "\n\tbackground " << _bgColor
		 << "\n" << ParticleManager::get()
		 << "\n}" 
		 << "\n" << _player // write out the player
		 << "\n" << *ObjectMeshManager::get() << endl; //the objectMeshManager has an overloaded stream out operator
													  // so it can run through the objects it manages and stream them out
}

/*------------------------------------------------------------
* EVENT HANDLERS for the View class
-------------------------------------------------------------- */
void View::OnCreate(){
	parseData(); //parse the environment.txt file

	_bHaveMultitex = false; //do we have multitexturing support?
	_bMultitex = false;	 //are we using multitex?

	SetSize(_iWidth, _iHeight); // set up the size, depthbits
	SetDepthBits(_iDepthBits);  // and position of the window
	SetPosition(_iX, _iY);

	//important - call base class to load extensions
	GLWindowEx::OnCreate();

	//does this driver have multitexture support
	_bHaveMultitex = glex::Supports("GL_ARB_multitexture");
	if(!_bHaveMultitex){
		//warn the user that multitexture support isn't found
		App::MsgPrintf("GL_ARB_multitexture not found!");
	}

	//if it supports multitexture, turn it on
	_bMultitex = _bHaveMultitex;

	//clear window first - might take a moment for image to load
	glClear(GL_COLOR_BUFFER_BIT);
	SwapBuffers();

	//viewport and depth testing
	glViewport(0,0, _iWidth, _iHeight);
	glEnable(GL_DEPTH_TEST);

	const GLfloat * bgCol = _bgColor.rgba(); //GLfloat array to temp hold bgColors values
	glClearColor(bgCol[0], bgCol[1], bgCol[2], bgCol[3]);

	//shade model
	glShadeModel(GL_SMOOTH);

	//remove back faces
	glEnable(GL_CULL_FACE);

	//materials
	_material1.create(Color::black(), Color(0.8f, 0.8f, 0.8f, 1.0f));

	//turn off the global ambient
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, Color::black().rgba());

	//fullscreen and wireframe mode
	SetFullscreen(_bFullscreen);
	glPolygonMode(GL_FRONT_AND_BACK, _bWireframe ? GL_LINE : GL_FILL);
	_bBoundingBox = false; //initially set the show bounding box to false

	SetCursor(CRCross); // set the cursor to a crosshair

	_deltaTime = (float)App::GetDeltaTime();
	_bBulletTime = false;
}

void View::OnDisplay(){
	//record the deltaTime
	_deltaTime  = (float)App::GetDeltaTime();
	if(_bBulletTime) _deltaTime *= 0.1f;

	// clear out the color and depth buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
		//update the player's position if he's moving
		if(_player.isMoving()){
			_player.move(_deltaTime);
		}

		if(_player.yPos() > 0.002) _player.applyGravity(_deltaTime);

		//get a security camera if there is one
		_currCam = ObjectMeshManager::get()->getNextSecurityCamera();
		if(_currCam != 0){
			glPushMatrix();
				_currCam->lookAt(_player.position()); // follow the player
				_currCam->showView(); // get the security camera's matrix

				glViewport(0,0, _currCam->getTexSize(), _currCam->getTexSize()); // resize the viewport

				renderScene(); // render the scene
				_currCam->capture(); // capture the image
			glPopMatrix();

			glViewport(0,0, _iWidth, _iHeight); // bring it back to it's old size

			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // clear the buffers
		}

		//get the camera matrix
		_player.showView();

		if(_bLeftDown) _player.shoot(getMouseVector()); // if shooting, get the bullet's vector

		renderScene();

	glPopMatrix();

	SwapBuffers();
}

/*-------------------------------------------------------------------
* render the scene - common functionality used by player and cameras
---------------------------------------------------------------------*/
void View::renderScene(){
	_material1.apply(); //apply basic material
	_player.draw();

	/* ------------------------- TESTING ----------------------------------*/
	if(_bBoundingBox){
		glDisable(GL_LIGHTING);
		glColor3f(1.0f, 0.0f, 0.0f);
		glLineWidth(5.0f); //set a thick lineWidth
		_player.drawBoundingBox(); // draw the player's bounding box
		glLineWidth(1.0f); // reset the lineWidth
		glEnable(GL_LIGHTING);
	}
	/*----------------------------------------------------------------------*/

	_material1.apply(); //apply basic material

	if(!_bGourand) glEnable(GL_TEXTURE_2D);
	ObjectMeshManager::get()->drawObjects(_bBoundingBox); //get the ObjectMeshManager to call the draw()
														  //function on the objects it manages
														  //_bBoundingBox is used for drawing the bounding boxes
														  // _bGourand is used to determine GL_TEXTURE_2D or not
	ParticleManager::get()->drawParticleEngines(); // draw any particle systems created
	glDisable(GL_TEXTURE_2D);
}

void View::OnIdle() {
	Redraw();
}

void View::OnResize(int w, int h){
	_iWidth = w;
	_iHeight = h;
}

void View::OnKeyboard(int key, bool down) {
	//key released
	switch(tolower(key)){
		case 'a':
			_player.isMoving(false);
			break;
		case 'z':
			_player.isMoving(false);
			break;
		default:
			break;
	}
	if (!down) return;
	//key pressed
	switch( tolower(key) ) {
		case 'a':
			_player.isMoving(true);
			_player.direction(-1);
			break;
		case 'z':
			_player.isMoving(true);
			_player.direction(1);
			break;
		case 'f':
			_bFullscreen = !_bFullscreen;
			SetFullscreen(_bFullscreen); // fullscreen mode
			break;
		case 'q':
			Close();
			break;
		case 'w':
			if(!_bGourand && !_bWireframe){
				_bGourand = true;
			} else if (_bGourand && !_bWireframe){
				_bWireframe = true;
				glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); //wireframe mode
			} else {
				_bGourand = _bWireframe = false;
				glPolygonMode(GL_FRONT_AND_BACK, GL_FILL); //wireframe mode
			}
			break;
		case 'd':
			writeData(); //write the world to the ouput file
			break;
		case 'c':
			_bBoundingBox = !_bBoundingBox; //write the world to the ouput file
			break;
		case 's':
			_player.changeYPos(_player.yPos() + 1.0);
			break;
		case 'x':
			_player.changeYPos(_player.yPos() - 1.0);
			break;
		case 'b':
			_bBulletTime = !_bBulletTime;
		default:
			break;
	}
}	

void View::OnMouseButton(MouseButton button, bool down) {
	switch(button){
		case MBLeft:
			_bLeftDown = down ? true : false;
			if(!_bLeftDown) _player.setIsShooting(false); // reset the player shooting var
			break;
		case MBRight:
			_bRightDown = down ? true : false;
			break;
		default:
			break;
	}
}
void View::OnMouseMove(int x, int y) {
	//static int temp_iX = 0, temp_iY = 0; //temp x and y pos
	static float xAngle = 0.0f;
	static float yAngle = 0.0f;
	static float angle_inc = 0.2f;

	if(_bRightDown){
		xAngle += angle_inc*(x-_iMousex)*_deltaTime; // incretement the xAngle based on how much we've moved left/right
		yAngle += angle_inc*(y-_iMousey)*_deltaTime; // incretement the yAngle based on how much we've moved up/down
		_player.turn(xAngle, yAngle); // turn the camera
	}

	_iMousex = x;
	_iMousey = y;
}

/*------------------------------------------------------------
* get a Vector3d from the mouse x,y position. first call 
* gluUnProject using 0.1 (near plane) for the z pos, then
* call it again using the far plane for the z pos. subtract
* between the two = xyz direction for the ray
--------------------------------------------------------------*/
Vector3d View::getMouseVector(){
	Vector3d nearTemp; // temp vector for the near plane
	Vector3d farTemp; // temp vector for the far plane
	GLdouble modelMatrix[16]; // temp array to hold the model matrix
	GLdouble projMatrix[16]; // temp array to hold the projection matrix
	GLint viewport[4]; // temp array to hold the viewport details
	glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix); // get the modelMatrix
	glGetDoublev(GL_PROJECTION_MATRIX, projMatrix); // get the projection matrix
	glGetIntegerv(GL_VIEWPORT, viewport); // get the viewport details
	gluUnProject(_iMousex, _iMousey, 0.1,
				 modelMatrix, projMatrix, viewport,
				 &nearTemp._x, &nearTemp._y, &nearTemp._z); // call gluUnProject on the near plane
	gluUnProject(_iMousex, _iMousey, _player.getFarPlane(),
				 modelMatrix, projMatrix, viewport,
				 &farTemp._x, &farTemp._y, &farTemp._z); // call gluUnProject on the far plane
	return nearTemp-farTemp; // return the vector
}