#pragma once

#include "GXBase.h"
#include "Color.h"
#include "Materials.h"
#include "ObjectMeshManager.h"
#include "TextureManager.h"
#include "ParticleManager.h"
#include "Player.h"
#include "SecurityCamera.h"
//for parsing
#include <fstream>
#include <string>

using namespace gxbase;
using namespace std;

class View : public GLWindowEx{
private:
	//opengl objects
	Materials _material1;
	Player _player;
	SecurityCamera * _currCam;

	//variables for multitexturing
	bool _bHaveMultitex; //do we have multitexturing support?
	bool _bMultitex;	 //are we using multitex?

	//simulation data
	int _iWidth, _iHeight, _iDepthBits, _iX, _iY;
	int _iMousex, _iMousey; // ints to hold the mouse position
	bool _bLeftDown, _bRightDown;
	bool _bFullscreen, _bWireframe, _bGourand, _bBoundingBox, _bBulletTime;
	float _deltaTime; // hold the deltaTime between calls
	Color _bgColor;

	/* ---------------------------------------------------------------
	* parse the environment file to get the relevant data needed to set up the 
	* MyWindow class. attributes are the name of the attribute, followed by
	* a white space, followed by a '(' bracket, followed by the parameters, 
	* followed by a ')' braket or
	* the name of the attribute, followed by a '=' sign, a white space, then
	* the parameter. multiple parameters are inside '(', ')' brackets.
	* multiple parameters are seperated by a ','.
	* return a bool based on the success of the parse - false fail, true succeed
	------------------------------------------------------------------*/
	bool parseData();
	void writeData(); // write the data back out to file

public:
	View();
	~View();

	/*------------------------------------------------------------
	* get a Vector3d from the mouse x,y position. first call 
	* gluUnProject using 0.1 (near plane) for the z pos, then
	* call it again using the far plane for the z pos. subtract
	* between the two = xyz direction for the ray
	--------------------------------------------------------------*/
	Vector3d getMouseVector();
	void OnCreate();
	void OnDisplay();
	void renderScene(); // render the scene - common functionality used by player and cameras
	void OnIdle();
	void OnResize(int w, int h);
	void OnKeyboard(int key, bool down);
	void OnMouseButton(MouseButton button, bool down);
	void OnMouseMove(int x, int y);
};