#include "Window.h"

/*------------------------------------------
* constructor - set up the window material
--------------------------------------------*/
Window::Window():_bIsBroken(0){
	_material.create(Color::black(), Color(0.0f,0.0f,0.4f,0.3f));
}

/*-------------------------------------
* the draw function - calls on the
* object to draw
---------------------------------------*/
void Window::draw() const{
	glPushMatrix();
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); // enable blending

		_material.apply();

		glTranslated(xPos(), yPos(), zPos());

		if(!_bIsBroken) _unBrokenWindow.draw();
		else _brokenWindow.draw();

		glDisable(GL_BLEND);
	glPopMatrix();
}

/*-------------------------------------------------------------------------
* the read function - gets passed the istream from the ObjectMeshManager
---------------------------------------------------------------------------*/
void Window::read(istream &fin){
	char buffer[256], ch;
	float w, h, d; // width, height
	
	// create two array, one for the texture strings for each side
	// and one for the corresponding number variables in this object
	// (need to make arrays of pointers to the variables themselves)
	string * sStreamTexNames[] = {&_sUnBrokenTexture, &_sBrokenTexture};
	GLuint * iTexNumArray[] = {&_iUnBrokenTexture, &_iBrokenTexture};

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> w; // read in the width
	fWidth(w); //set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> h; // read in the height
	fHeight(h); // set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> d; // read in the depth
	fDepth(d); // set it

	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> position(); // read in the position

	for(int i=0; i<2; i++){
		fin.get(buffer, sizeof(buffer), '=');
		fin >> ch >> ch >> *sStreamTexNames[i] >> ch; // read in the six textures
		*iTexNumArray[i] = getTexture(*sStreamTexNames[i], true); // assign the texture number
	}

	/*--------------------------------------------------------*/
	string tempStr;
	fin.get(buffer, sizeof(buffer), '=');
	fin >> ch >> ch >> tempStr >> ch; // read in the id
	setID(tempStr); // set it
	/*--------------------------------------------------------*/

	void create(const float w, const float h, const GLuint iTextureNum);
	_unBrokenWindow.create(fWidth(), fHeight(), fDepth(),
				  _iUnBrokenTexture, _iUnBrokenTexture, _iUnBrokenTexture, _iUnBrokenTexture, _iUnBrokenTexture, _iUnBrokenTexture,
				  false);
	_brokenWindow.create(fWidth(), fHeight(), fDepth(),
				  _iBrokenTexture, _iBrokenTexture, _iBrokenTexture, _iBrokenTexture, _iBrokenTexture, _iBrokenTexture,
				  false);

	setupBoundingBox(); // create the bounding box
}

/*---------------------------------------------
* the write function - stream out the data
-----------------------------------------------*/
ostream& Window::write(ostream &out){
	out << "Window {"
		 << "\n\twidth= " << fWidth()
		 << "\n\theight= " << fHeight()
		 << "\n\tdepth= " << fDepth()
		 << "\n\tposition= " << position()
		 << "\n\tunbrokenTexture= \"" << _sUnBrokenTexture << " \""
		 << "\n\tbrokenTexture= \"" << _sBrokenTexture << " \""
		 << "\n\tid= \"" << getID() << " \""
		 << "\n}" << endl;
	return out;
}

/*---------------------------------------------
* set the bounding box - calls Object::setBoundingBox()
-----------------------------------------------*/
void Window::setupBoundingBox(){
	double tempw = xPos() + fWidth() * 0.5;
	double temph = yPos() + fHeight() * 0.5;
	double tempd = zPos() + fDepth() * 0.5;
	setBoundingBox(fWidth(), fHeight(), fDepth(), tempw, temph, tempd);
}